/*
 * AVL_Tree.h    (c) Tommy Olsson, IDA, 2007-05-02
 *
 * operator<< frutstts vara definierad fr Comparable.
 */
#ifndef AVLTREE_INC
#define AVLTREE_INC
#include <iosfwd>
#include <stdexcept>

class avl_tree_error : public std::logic_error 
{
public:
   explicit avl_tree_error(const std::string& what_arg)
      : std::logic_error(what_arg) {}
};

template<typename Comparable> class AVL_Tree_Node;

template<typename Comparable>
class AVL_Tree 
{
public:
   AVL_Tree();
   AVL_Tree(const AVL_Tree&);
   ~AVL_Tree();
   AVL_Tree& operator=(AVL_Tree rhs);

   void        insert(const Comparable& element);
   void        remove(const Comparable& element);   
   bool        member(const Comparable&) const;
   Comparable& retrieve();

   Comparable& find(const Comparable& element) const;
   Comparable& find_min() const;
   Comparable& find_max() const;

   void clear();
   bool empty() const;

   void print(std::ostream&) const;
   void print_tree(std::ostream&) const;

   void swap(AVL_Tree& other);

private:
   typedef AVL_Tree_Node<Comparable>  Node;
   typedef AVL_Tree_Node<Comparable>* Node_Pointer;

   Node_Pointer         root_;
   mutable Node_Pointer current_;
};

template<typename Comparable>
void swap(AVL_Tree<Comparable>&, AVL_Tree<Comparable>&);

template<typename Comparable>
std::ostream& operator<<(std::ostream&, const AVL_Tree<Comparable>&);

/* Changed file type to .cc to make emacs launch c++ mode for the file.
 * Also, if you're testing this is VC, put everything in the header file
 * (that is, in this file). */
#include "AVL_Tree.cc"

#endif
